


#define MY_DEBUG
#define MY_DEBUG_VERBOSE_SIGNING
#define MY_SIGNING_ATSHA204
//#define MY_SIGNING_REQUEST_SIGNATURES


#define MY_OTA_FIRMWARE_FEATURE
#define MY_OTA_FLASH_JDECID 0xEF40
#define OTA_WAIT_PERIOD 300
#define FIRMWARE_MAX_REQUESTS 10
#define MY_OTA_RETRY 10


#define MY_RADIO_RFM69


#define MY_RFM69_NEW_DRIVER
#define MY_RFM69_FREQUENCY RFM69_868MHZ


#define MY_DEFAULT_ERR_LED_PIN 4  // Error led pin
#define MY_DEFAULT_RX_LED_PIN  5  // Receive led pin
#define MY_DEFAULT_TX_LED_PIN  6
#define MY_WITH_LEDS_BLINKING_INVERSE
#define MY_NODE_ID 5


#include <MySensors.h>

#include <Wire.h>
#include <Adafruit_MCP4725.h>
#include <avr/wdt.h>

Adafruit_MCP4725 dac;

#define DAC_RESOLUTION    (9)

#define CHILD_ID_LIGHT 1

#define EPROM_LIGHT_STATE 1
#define EPROM_DIMMER_LEVEL 2

#define LIGHT_OFF 0
#define LIGHT_ON 1

#define SN "Dimable Light"
#define SV "4.04"


unsigned long previousMillis = 0;        // will store last time LED was updated

// constants won't change:
const long interval = 100000;           // interval at which to blink (milliseconds)


int16_t LastLightState = LIGHT_OFF;
int16_t LastDimValue = 0;

MyMessage lightMsg(CHILD_ID_LIGHT, V_LIGHT);
MyMessage dimmerMsg(CHILD_ID_LIGHT, V_DIMMER);


void before()
{
  wdt_disable(); 
  wdt_enable(WDTO_8S);
}

void setup()
{
  dac.begin(0x60);
  int LightState = loadState(EPROM_LIGHT_STATE);
  if (LightState <= 1) {
    LastLightState = LightState;
    int DimValue = loadState(EPROM_DIMMER_LEVEL);
    if ((DimValue > 0) && (DimValue <= 100)) {
      DimValue = 50;
      LastDimValue = DimValue;
    }
  }

  SetCurrentState2Hardware();
}

void presentation()
{
  sendSketchInfo(SN, SV,1);
  present(CHILD_ID_LIGHT, S_DIMMER );
}

void loop()
{
  wdt_reset();

  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    sendHeartbeat();
  }
}

void receive(const MyMessage &message)
{
  if (message.type == V_LIGHT) {
    int lstate = atoi( message.data );
    if ((lstate < 0) || (lstate > 1)) {
      return;
    }
    LastLightState = lstate;
    saveState(EPROM_LIGHT_STATE, LastLightState);
    if ((LastLightState == LIGHT_ON) && (LastDimValue == 0)) {
      LastDimValue = 100;
      saveState(EPROM_DIMMER_LEVEL, LastDimValue);
    }
  } else if (message.type == V_DIMMER) {
    int dimvalue = atoi( message.data );
    if ((dimvalue < 0) || (dimvalue > 100)) {
      return;
    }
    if (dimvalue == 0) {
      LastLightState = LIGHT_OFF;
    } else {
      LastLightState = LIGHT_ON;
      LastDimValue = dimvalue;
      saveState(EPROM_DIMMER_LEVEL, LastDimValue);
    }
  } else {
    return;
  }
  SetCurrentState2Hardware();
}

void SetCurrentState2Hardware()
{
  if (LastLightState == LIGHT_OFF) {
    dac.setVoltage(0, false);
  }
  else {
    if (LastDimValue == 0)
    {
      dac.setVoltage(0, false);
    }
    else {
      if (LastDimValue < 100) {
        dac.setVoltage(map(LastDimValue, 0, 99, 70, 1758), false);
      }
      else {
        dac.setVoltage(1759, false);
      }
    }
  }
  SendCurrentState2Controller();
}

void SendCurrentState2Controller()
{
  if ((LastLightState == LIGHT_OFF) || (LastDimValue == 0)) {
    send(dimmerMsg.set((int16_t)0), true);
  } else {
    send(dimmerMsg.set(LastDimValue), true);
  }
}
